@@ -20,9 +20,9 @@ use SDL::App::FPS::Demo;
 # fill in here options or use Getopt::Long for command line
 my $options = { width => 640, height => 480, max_fps => 40};
 
-print "Kcirb v0.01 (C) 2002 by Tels <http://Bloodgate.com/>\n\n";
+print "SDL::App::FPS Demo v0.01 (C) 2002 by Tels <http://Bloodgate.com/>\n\n";
 
-my $app = SDL::App::FPS::Kcirb->new( $options );
+my $app = SDL::App::FPS::Demo->new( $options );
 $app->main_loop();
 
 print "Running time was ", int($app->now() / 10)/100, " seconds\n";
